@@ -21,6 +21,7 @@
 #'  \item{"selected"}{ : Optional. Integer/Character. Initial id selection. Defaut to NULL}
 #'  \item{"style"}{ : Optional. Character. HTML style of list. Default to 'width: 150px; height: 26px'.}
 #'  \item{"useLabels"}{ : Optional. Boolean. Use labels instead of id ? Default to TRUE.}
+#'  \item{"main"}{ : Optional. Default to "Select by id"}
 #'}
 #'@param selectedBy : Custom option. Character or a named list. Add a multiple selection based on column of node data.frame creating an HTML select element. Not available for DOT and Gephi.
 #'\itemize{
@@ -30,6 +31,7 @@
 #'  \item{"style"}{ : Optional. Character. HTML style of list. Default to 'width: 150px; height: 26px'.}
 #'  \item{"multiple"}{ : Optional. Boolean. Default to FALSE. If TRUE, you can affect multiple groups per nodes using a comma ("gr1,gr2")}
 #'  \item{"hideColor"}{ : Optional. String. Color for hidden nodes/edges. Use a rgba definition. Defaut to rgba(200,200,200,0.5)}
+#'  \item{"main"}{ : Optional. Default to "Select by variable"}
 #'}
 #'@param collapse : Custom option. Just a Boolean, or a named list. Collapse / Uncollapse nodes using double-click. In dev.
 #'\itemize{
@@ -325,10 +327,10 @@ visOptions <- function(graph,
     #############################
     # nodesIdSelection
     #############################
-    idselection <- list(enabled = FALSE, style = 'width: 150px; height: 26px', useLabels = TRUE)
+    idselection <- list(enabled = FALSE, style = 'width: 150px; height: 26px', useLabels = TRUE, main = "Select by id")
     if(is.list(nodesIdSelection)){
-      if(any(!names(nodesIdSelection)%in%c("enabled", "selected", "style", "values", "useLabels"))){
-        stop("Invalid 'nodesIdSelection' argument. List can have 'enabled', 'selected', 'style', 'values', 'useLabels'")
+      if(any(!names(nodesIdSelection)%in%c("enabled", "selected", "style", "values", "useLabels", "main"))){
+        stop("Invalid 'nodesIdSelection' argument. List can have 'enabled', 'selected', 'style', 'values', 'useLabels', 'main'")
       }
       if("selected"%in%names(nodesIdSelection)){
         if(any(class(graph) %in% "visNetwork")){
@@ -344,6 +346,10 @@ visOptions <- function(graph,
         idselection$enabled <- TRUE
       }
       
+      if("main"%in%names(nodesIdSelection)){
+        idselection$main <- nodesIdSelection$main
+      }
+      
       if("useLabels"%in%names(nodesIdSelection)){
         idselection$useLabels <- nodesIdSelection$useLabels
       }else if(any(class(graph) %in% "visNetwork_Proxy")){
@@ -387,8 +393,8 @@ visOptions <- function(graph,
     
     if(!is.null(selectedBy)){
       if(is.list(selectedBy)){
-        if(any(!names(selectedBy)%in%c("variable", "selected", "style", "values", "multiple", "hideColor"))){
-          stop("Invalid 'selectedBy' argument. List can have 'variable', 'selected', 'style', 'values', 'multiple', 'hideColor'")
+        if(any(!names(selectedBy)%in%c("variable", "selected", "style", "values", "multiple", "hideColor", "main"))){
+          stop("Invalid 'selectedBy' argument. List can have 'variable', 'selected', 'style', 'values', 'multiple', 'hideColor', 'main'")
         }
         if("selected"%in%names(selectedBy)){
           byselection$selected <- as.character(selectedBy$selected)
@@ -404,6 +410,12 @@ visOptions <- function(graph,
         
         byselection$variable <- selectedBy$variable
         
+        if("main" %in% names(selectedBy)){
+          byselection$main <- selectedBy$main
+        } else {
+          byselection$main <- paste0("Select by ", selectedBy$variable)
+        }
+ 
         if("style"%in%names(selectedBy)){
           byselection$style <- selectedBy$style
         }else if(any(class(graph) %in% "visNetwork_Proxy")){
@@ -419,6 +431,8 @@ visOptions <- function(graph,
       }else if(is.character(selectedBy)){
         byselection$variable <- selectedBy
         
+        byselection$main <- paste0("Select by ", selectedBy)
+        
         if(any(class(graph) %in% "visNetwork_Proxy")){
           byselection$style <- NULL
           byselection$multiple <- NULL
