@@ -4,6 +4,7 @@
 #' 
 #'@param graph : a \code{\link{visNetworkProxy}}  object
 #'@param id : vector of id, edges to remove
+#'@param legend : Boolean. Remove edges on legend ? Default to FALSE
 #'
 #'@seealso \link{visNodes} for nodes options, \link{visEdges} for edges options, \link{visGroups} for groups options, 
 #'\link{visLegend} for adding legend, \link{visOptions} for custom option, \link{visLayout} & \link{visHierarchicalLayout} for layout, 
@@ -22,7 +23,7 @@
 #' 
 #'@export
 #'@references See online documentation \url{http://datastorm-open.github.io/visNetwork/}
-visRemoveEdges <- function(graph, id){
+visRemoveEdges <- function(graph, id, legend = FALSE){
 
   if(!any(class(graph) %in% "visNetwork_Proxy")){
     stop("Can't use visRemoveEdges with visNetwork object. Only within shiny & using visNetworkProxy")
@@ -34,7 +35,7 @@ visRemoveEdges <- function(graph, id){
     }
   }
   
-  data <- list(id = graph$id, rmid = id)
+  data <- list(id = graph$id, rmid = id, legend = legend)
   
   graph$session$sendCustomMessage("visShinyRemoveEdges", data)
 
